# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-04-05 21:07
from __future__ import unicode_literals

from decimal import Decimal
from django.db import migrations, models
import web_payments.logic
import web_payments.translation


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='QPayment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('variant', models.CharField(max_length=255)),
                ('status', models.CharField(choices=[('waiting', web_payments.translation._lazy_constant(('web_payments.translation.translation', 'gettext'), 'Waiting for confirmation', None)), ('preauth', web_payments.translation._lazy_constant(('web_payments.translation.translation', 'gettext'), 'Pre-authorized', None)), ('confirmed', web_payments.translation._lazy_constant(('web_payments.translation.translation', 'gettext'), 'Confirmed', None)), ('rejected', web_payments.translation._lazy_constant(('web_payments.translation.translation', 'gettext'), 'Rejected', None)), ('refunded', web_payments.translation._lazy_constant(('web_payments.translation.translation', 'gettext'), 'Refunded', None)), ('error', web_payments.translation._lazy_constant(('web_payments.translation.translation', 'gettext'), 'Error', None)), ('input', web_payments.translation._lazy_constant(('web_payments.translation.translation', 'gettext'), 'Input', None))], default='waiting', max_length=10)),
                ('message', models.TextField(blank=True, default='')),
                ('fraud_status', models.CharField(choices=[('unknown', web_payments.translation._lazy_constant(('web_payments.translation.translation', 'gettext'), 'Unknown', None)), ('accept', web_payments.translation._lazy_constant(('web_payments.translation.translation', 'gettext'), 'Passed', None)), ('reject', web_payments.translation._lazy_constant(('web_payments.translation.translation', 'gettext'), 'Rejected', None)), ('review', web_payments.translation._lazy_constant(('web_payments.translation.translation', 'gettext'), 'Review', None))], default='unknown', max_length=10)),
                ('fraud_message', models.TextField(blank=True, default='')),
                ('extra_data', models.TextField(blank=True, default='')),
                ('token', models.CharField(blank=True, default='', max_length=36)),
                ('transaction_id', models.CharField(blank=True, max_length=255)),
                ('currency', models.CharField(max_length=10)),
                ('total', models.DecimalField(decimal_places=8, default=Decimal('0.0'), max_digits=20)),
                ('captured_amount', models.DecimalField(decimal_places=8, default=Decimal('0.0'), max_digits=20)),
                ('billing_first_name', models.CharField(blank=True, max_length=256)),
                ('billing_last_name', models.CharField(blank=True, max_length=256)),
                ('billing_address_1', models.CharField(blank=True, max_length=256)),
                ('billing_address_2', models.CharField(blank=True, max_length=256)),
                ('billing_email', models.EmailField(blank=True, max_length=254)),
                ('billing_city', models.CharField(blank=True, max_length=256)),
                ('billing_postcode', models.CharField(blank=True, max_length=20)),
                ('billing_country_code', models.CharField(blank=True, max_length=2)),
                ('billing_country_area', models.CharField(blank=True, max_length=256)),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model, web_payments.logic.BasicPayment),
        ),
    ]
